@@ -36,41 +36,6 @@
  *
  **/
 
-static void
-async_result_free (gpointer data)
-{
-  GAsyncResultData *res = data;
-
-  if (res->error)
-    g_error_free (res->error);
-
-  g_object_unref (res->async_object);
-  
-  g_free (res);
-}
-
-void
-_g_queue_async_result (GAsyncResultData *result,
-		       gpointer          async_object,
-		       GError           *error,
-		       gpointer          user_data,
-		       GSourceFunc       source_func)
-{
-  GSource *source;
-
-  g_return_if_fail (G_IS_OBJECT (async_object));
-  
-  result->async_object = g_object_ref (async_object);
-  result->user_data = user_data;
-  result->error = error;
-
-  source = g_idle_source_new ();
-  g_source_set_priority (source, G_PRIORITY_DEFAULT);
-  g_source_set_callback (source, source_func, result, async_result_free);
-  g_source_attach (source, NULL);
-  g_source_unref (source);
-}
-
 /*************************************************************************
  *             fd source                                                 *
  ************************************************************************/
@@ -81,6 +46,7 @@ typedef struct
   GPollFD pollfd;
   GCancellable *cancellable;
   gulong cancelled_tag;
+  GObject *object;
 } FDSource;
 
 static gboolean 
@@ -93,7 +59,7 @@ fd_source_prepare (GSource *source,
   return g_cancellable_is_cancelled (fd_source->cancellable);
 }
 
-static gboolean 
+static gboolean
 fd_source_check (GSource *source)
 {
   FDSource *fd_source = (FDSource *)source;
@@ -110,24 +76,31 @@ fd_source_dispatch (GSource     *source,
 
 {
   GFDSourceFunc func = (GFDSourceFunc)callback;
+  GFDSourceObjectFunc func2 = (GFDSourceObjectFunc)callback;
   FDSource *fd_source = (FDSource *)source;
 
   g_warn_if_fail (func != NULL);
 
-  return (*func) (user_data, fd_source->pollfd.revents, fd_source->pollfd.fd);
+  if (fd_source->object)
+    return (*func2) (fd_source->object, fd_source->pollfd.revents, user_data);
+  else
+    return (*func) (user_data, fd_source->pollfd.revents, fd_source->pollfd.fd);
 }
 
-static void 
+static void
 fd_source_finalize (GSource *source)
 {
   FDSource *fd_source = (FDSource *)source;
 
   if (fd_source->cancelled_tag)
-    g_signal_handler_disconnect (fd_source->cancellable,
-				 fd_source->cancelled_tag);
+    g_cancellable_disconnect (fd_source->cancellable,
+			      fd_source->cancelled_tag);
 
   if (fd_source->cancellable)
     g_object_unref (fd_source->cancellable);
+
+  if (fd_source->object)
+    g_object_unref (fd_source->object);
 }
 
 static GSourceFuncs fd_source_funcs = {
@@ -147,9 +120,10 @@ fd_source_cancelled_cb (GCancellable *cancellable,
 }
 
 GSource *
-_g_fd_source_new (int           fd,
-		  gushort       events,
-		  GCancellable *cancellable)
+_g_fd_source_new_with_object (GObject      *object,
+			      int           fd,
+			      gushort       events,
+			      GCancellable *cancellable)
 {
   GSource *source;
   FDSource *fd_source;
@@ -159,17 +133,27 @@ _g_fd_source_new (int           fd,
 
   if (cancellable)
     fd_source->cancellable = g_object_ref (cancellable);
-  
+
+  if (object)
+    fd_source->object = g_object_ref (object);
+
   fd_source->pollfd.fd = fd;
   fd_source->pollfd.events = events;
   g_source_add_poll (source, &fd_source->pollfd);
 
   if (cancellable)
     fd_source->cancelled_tag =
-      g_signal_connect_data (cancellable, "cancelled",
+      g_cancellable_connect (cancellable,
 			     (GCallback)fd_source_cancelled_cb,
-			     NULL, NULL,
-			     0);
-  
+			     NULL, NULL);
+
   return source;
 }
+
+GSource *
+_g_fd_source_new (int           fd,
+		  gushort       events,
+		  GCancellable *cancellable)
+{
+  return _g_fd_source_new_with_object (NULL, fd, events, cancellable);
+}
